@@ -1746,6 +1746,14 @@ class RegisterModelVersionResponse(BaseModel):
     model_version: int
     artifact: Any
     aliases: List[str]
+    created_by: str
+    created_at: Optional[datetime] = None
+
+
+class RegisterModelArtifactResponse(BaseModel):
+    artifact_id: str
+    path: str
+    spec: Any
     metadata: Mapping[str, Any]
     created_by: str
     created_at: Optional[datetime] = None
@@ -1766,7 +1774,6 @@ class GetRegisteredModelResponse(BaseModel):
 class GetRegisteredModelVersionResponse(BaseModel):
     model_id: str
     model_name: str
-    metadata: Mapping[str, Any]
     created_by: str
     created_at: Optional[datetime] = None
     updated_at: Optional[datetime] = None
