@@ -1,4 +1,5 @@
 import os
+import pickle
 import tempfile
 from typing import Any, Callable, List, Mapping, NamedTuple, Optional, Tuple, Type
 
@@ -6,8 +7,9 @@ from google.protobuf import struct_pb2
 
 from chalk._gen.chalk.models.v1 import model_artifact_pb2 as _model_artifact_pb2
 from chalk._gen.chalk.models.v1.model_artifact_pb2 import ModelFile
+from chalk._gen.chalk.server.v1.model_registry_pb2 import RunCriterion, RunCriterionDirection
 from chalk.ml.model_file_transfer import FileInfo
-from chalk.ml.utils import ModelAttributeExtractor, ModelEncoding, ModelType
+from chalk.ml.utils import ModelAttributeExtractor, ModelEncoding, ModelRunCriterion, ModelType
 
 ModelSchemaType = List[Tuple[List[int], Any]]
 
@@ -80,14 +82,18 @@ MODEL_SERIALIZERS = {
 
 
 class ModelSerializer:
-    def __init__(self, model: Any, model_type: ModelType):
+    def __init__(self, model: Any, model_type: Optional[ModelType]):
         self._temp_files: List[str] = []
-        self._model = model
-        self._model_type = model_type
+        self.model = model
+        self.model_type = model_type if model_type is not None else ModelAttributeExtractor.infer_model_type(model)
+
+        if self.model_type is None:
+            raise ValueError("Unable to infer model type from object and no type given.")
+
         super().__init__()
 
     @classmethod
-    def from_model(cls, model: Any, model_type: ModelType) -> "ModelSerializer":
+    def from_model(cls, model: Any, model_type: Optional[ModelType] = None) -> "ModelSerializer":
         return cls(model=model, model_type=model_type)
 
     def __enter__(self) -> "ModelSerializer":
@@ -100,7 +106,8 @@ class ModelSerializer:
         return False
 
     def serialize(self) -> Tuple[str, ModelEncoding]:
-        return self.serialize_model(self._model, self._model_type)
+        assert self.model_type is not None, "Could not determine model type. Please set parameter: model_type."
+        return self.serialize_model(self.model, self.model_type)
 
     def serialize_model(self, model: Any, model_type: ModelType) -> tuple[str, ModelEncoding]:
         if model_type not in MODEL_SERIALIZERS:
@@ -127,7 +134,8 @@ class ModelSerializer:
             raise ImportError(error_msg)
 
     def get_dependencies(self) -> Optional[List[str]]:
-        return (MODEL_SERIALIZERS[self._model_type].dependency_fn or (lambda: None))()
+        assert self.model_type is not None, "Could not determine model type. Please set parameter: model_type."
+        return (MODEL_SERIALIZERS[self.model_type].dependency_fn or (lambda: None))()
 
     def cleanup(self):
         import shutil
@@ -173,16 +181,33 @@ class ModelSerializer:
                 metadata_converted[k] = converted_v
         return metadata_converted
 
-    @staticmethod
     def infer_input_output_schemas(
-        model: Any, model_type: ModelType
+        self, model: Optional[Any] = None, model_type: Optional[ModelType] = None
     ) -> Tuple[Optional[ModelSchemaType], Optional[ModelSchemaType]]:
+        if model is None:
+            model = self.model
+
+        if model_type is None:
+            assert self.model_type is not None, "Could not determine model type. Please set parameter: model_type."
+            model_type = self.model_type
+
         serializer_config = MODEL_SERIALIZERS[model_type]
         if serializer_config.schema_fn:
             return serializer_config.schema_fn(model)
         else:
             return None, None
 
+    def pickle_object(self, object: Any, object_name: str) -> str:
+        tmp_dir = tempfile.mkdtemp()
+        tmp_file_path = os.path.join(tmp_dir, f"{object_name}.pkl")
+
+        with open(tmp_file_path, "wb") as file:
+            pickle.dump(object, file)
+
+        self._temp_files.append(tmp_file_path)
+
+        return tmp_file_path
+
     @staticmethod
     def build_tabular_schema(column_dtypes: Mapping[str, Any]) -> _model_artifact_pb2.TabularSchema:
         import pyarrow as pa
@@ -261,3 +286,26 @@ class ModelSerializer:
             raise ValueError(f"Invalid empty schema.")
 
         return model_schema
+
+    @staticmethod
+    def convert_run_criterion_to_proto(
+        run_name: Optional[str], criterion: Optional[ModelRunCriterion]
+    ) -> Optional[RunCriterion]:
+        if run_name is None:
+            return None
+
+        if criterion is None:
+            return RunCriterion(run_id=run_name)
+
+        if criterion.direction == "max":
+            return RunCriterion(
+                run_id=run_name, metric=criterion.metric, direction=RunCriterionDirection.RUN_CRITERION_DIRECTION_MAX
+            )
+        elif criterion.direction == "min":
+            return RunCriterion(
+                run_id=run_name, metric=criterion.metric, direction=RunCriterionDirection.RUN_CRITERION_DIRECTION_MIN
+            )
+        else:
+            raise ValueError(
+                f"Unable to create model training run criterion. Unrecognized direction: {criterion.direction}."
+            )
