@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from chalk.ml.model_file_transfer import HFSourceConfig, LocalSourceConfig, S3SourceConfig, SourceConfig
 from chalk.ml.model_reference import ModelReference
-from chalk.ml.utils import ModelEncoding, ModelType
+from chalk.ml.utils import ModelEncoding, ModelRunCriterion, ModelType
 
 __all__ = (
     "ModelType",
@@ -12,4 +12,5 @@ __all__ = (
     "LocalSourceConfig",
     "S3SourceConfig",
     "HFSourceConfig",
+    "ModelRunCriterion",
 )
