@@ -4,15 +4,20 @@ from os import PathLike
 from typing import Any, Dict, Optional, Union, overload
 
 from chalk.sql._internal.incremental import IncrementalSettings
+from chalk.sql._internal.integrations.athena import AthenaSourceImpl
 from chalk.sql._internal.integrations.bigquery import BigQuerySourceImpl
+from chalk.sql._internal.integrations.clickhouse import ClickhouseSourceImpl
 from chalk.sql._internal.integrations.cloudsql import CloudSQLSourceImpl
 from chalk.sql._internal.integrations.databricks import DatabricksSourceImpl
+from chalk.sql._internal.integrations.dynamodb import DynamoDBSourceImpl
 from chalk.sql._internal.integrations.mysql import MySQLSourceImpl
 from chalk.sql._internal.integrations.postgres import PostgreSQLSourceImpl
 from chalk.sql._internal.integrations.redshift import RedshiftSourceImpl
 from chalk.sql._internal.integrations.snowflake import SnowflakeSourceImpl
+from chalk.sql._internal.integrations.spanner import SpannerSourceImpl
 from chalk.sql._internal.integrations.sqlite import SQLiteSourceImpl
 from chalk.sql._internal.integrations.trino import TrinoSourceImpl
+from chalk.sql._internal.sql_file_resolver import make_sql_file_resolver
 from chalk.sql._internal.sql_source_group import SQLSourceGroup
 from chalk.sql.finalized_query import FinalizedChalkQuery
 from chalk.sql.protocols import (
@@ -83,6 +88,7 @@ def SnowflakeSource(
     db: str | None = ...,
     schema: str | None = ...,
     role: str | None = ...,
+    private_key_b64: str | None = ...,
     engine_args: Optional[Dict[str, Any]] = ...,
 ) -> BaseSQLSourceProtocol:
     """You can also configure the integration directly using environment
@@ -141,6 +147,7 @@ def SnowflakeSource(
     db: Optional[str] = None,
     schema: Optional[str] = None,
     role: Optional[str] = None,
+    private_key_b64: Optional[str] = None,
     engine_args: Optional[Dict[str, Any]] = None,
 ) -> BaseSQLSourceProtocol:
     """Create a Snowflake data source. SQL-based data sources
@@ -161,6 +168,7 @@ def SnowflakeSource(
         db=db,
         schema=schema,
         role=role,
+        private_key_b64=private_key_b64,
         engine_args=engine_args,
     )
 
@@ -589,7 +597,7 @@ def RedshiftSource(
     Parameters
     ----------
     name
-        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
+        The name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
     host
         Name of host to connect to.
     db
@@ -599,7 +607,7 @@ def RedshiftSource(
     password
         The password for the Redshift database.
     port
-        Port number to use.
+        The port number to use.
     engine_args
         Additional arguments to use when constructing the SQLAlchemy engine.
 
@@ -647,6 +655,137 @@ def RedshiftSource(
     return RedshiftSourceImpl(host, db, user, password, name, port, engine_args=engine_args)
 
 
+@overload
+def SpannerSource() -> BaseSQLSourceProtocol:
+    """If you have only one Spanner connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = SpannerSource()
+    """
+    ...
+
+
+@overload
+def SpannerSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
+    """If you have only one Spanner integration, there's no need to provide
+    a distinguishing name.
+
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = SpannerSource(name="RISK")
+    """
+    ...
+
+
+@overload
+def SpannerSource(
+    *,
+    name: str | None = None,
+    project: str | None = None,
+    instance: str | None = None,
+    database: str | None = None,
+    credentials_base64: str | None = None,
+    emulator_host: str | None = None,
+    engine_args: Dict[str, Any] | None = None,
+) -> BaseSQLSourceProtocol:
+    """You can also configure the integration directly using environment
+    variables on your local machine or from those added through the
+    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
+
+    Parameters
+    ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
+    project
+        The name of the GCP project for the Spanner instance.
+    instance
+        The name of the Spanner instance.
+    database
+        The name of the database in the Spanner instance.
+    credentials_base64
+        The credentials to use to connect, encoded as a base64 string.
+    emulator_host
+        Location of Spanner emulator, if desired.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> import os
+    >>> source = SpannerSource(
+    ...     project=os.getenv("SPANNER_PROJECT"),
+    ...     instance=os.getenv("SPANNER_INSTANCE"),
+    ...     database=os.getenv("SPANNER_DATABASE"),
+    ...     credentials_base64=os.getenv("SPANNER_CREDENTIALS_BASE64"),
+    ... )
+    """
+    ...
+
+
+def SpannerSource(
+    name: str | None = None,
+    project: str | None = None,
+    instance: str | None = None,
+    database: str | None = None,
+    credentials_base64: str | None = None,
+    emulator_host: str | None = None,
+    engine_args: Dict[str, Any] | None = None,
+) -> BaseSQLSourceProtocol:
+    """Create a Spanner data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
+    return SpannerSourceImpl(
+        name=name,
+        project=project,
+        instance=instance,
+        database=database,
+        credentials_base64=credentials_base64,
+        emulator_host=emulator_host,
+        engine_args=engine_args,
+    )
+
+
 @overload
 def BigQuerySource() -> BaseSQLSourceProtocol:
     """If you have only one BigQuery connection that you'd like
@@ -1169,23 +1308,456 @@ def DatabricksSource(
     )
 
 
-__all__ = [
+@overload
+def DynamoDBSource() -> BaseSQLSourceProtocol:
+    """If you have only one DynamoDB connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+    Examples
+    --------
+    >>> source = DynamoDBSource()
+    """
+    ...
+
+
+@overload
+def DynamoDBSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
+    """If you have only one DynamoDB integration, there's no need to provide
+    a distinguishing name.
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        The name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = DynamoDBSource(name="RISK")
+    """
+    ...
+
+
+@overload
+def DynamoDBSource(
+    *,
+    name: str | None = None,
+    aws_client_id_override: str | None = None,
+    aws_client_secret_override: str | None = None,
+    aws_role_arn_override: str | None = None,
+    aws_region_override: str | None = None,
+    endpoint_override: str | None = None,
+    engine_args: Dict[str, Any] | None = None,
+) -> BaseSQLSourceProtocol:
+    """You can also configure the integration directly using environment
+    variables on your local machine or from those added through the
+    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
+
+    Parameters
+    ----------
+    name
+        The name of the integration. Not required unless if this SQL Source
+        is used within SQL File Resolvers.
+    aws_client_id_override
+        Optionally override the credentials using an AWS client id, must be
+        provided with a client secret override.
+    aws_client_secret_override
+        Optionally override the credentials using an AWS client secret, must
+        be provided with a client id override.
+    aws_role_arn_override
+        Optionally override the credentials using an AWS role ARN.
+    aws_region_override
+        The AWS region code to connect to, such as `"us-east-1"`.
+    endpoint_override
+        Override for the DynamoDB endpoint if hosted elsewhere.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = DynamoDBSource(name="RISK")
+    """
+    ...
+
+
+def DynamoDBSource(
+    name: str | None = None,
+    aws_client_id_override: str | None = None,
+    aws_client_secret_override: str | None = None,
+    aws_role_arn_override: str | None = None,
+    aws_region_override: str | None = None,
+    endpoint_override: str | None = None,
+    engine_args: Dict[str, Any] | None = None,
+) -> BaseSQLSourceProtocol:
+    """Create a DynamoDB data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details. DynamoDBSources can be queried via PartiQL SQL
+    resolvers.
+
+    You may override the ambient AWS credentials by providing
+    either a client ID and secret, or a role ARN.
+    """
+    return DynamoDBSourceImpl(
+        name=name,
+        aws_client_id_override=aws_client_id_override,
+        aws_client_secret_override=aws_client_secret_override,
+        aws_role_arn_override=aws_role_arn_override,
+        aws_region_override=aws_region_override,
+        endpoint_override=endpoint_override,
+        engine_args=engine_args,
+    )
+
+
+@overload
+def AthenaSource() -> BaseSQLSourceProtocol:
+    """If you have only one Athena connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+    Examples
+    --------
+    >>> source = AthenaSource()
+    """
+    ...
+
+
+@overload
+def AthenaSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
+    """If you have only one Athena integration, there's no need to provide
+    a distinguishing name.
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        The name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = AthenaSource(name="RISK")
+    """
+    ...
+
+
+@overload
+def AthenaSource(
+    *,
+    name: str | None = None,
+    aws_region: str | None = None,
+    aws_access_key_id: str | None = None,
+    aws_access_key_secret: str | None = None,
+    s3_staging_dir: str | None = None,
+    catalog_name: str | None = None,
+    schema_name: str | None = None,
+    role_arn: str | None = None,
+    work_group: str | None = None,
+    engine_args: Dict[str, Any] | None = None,
+) -> BaseSQLSourceProtocol:
+    """You can also configure the integration directly using environment
+    variables on your local machine or from those added through the
+    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
+
+    Parameters
+    ----------
+    name
+        The name of the integration. Not required unless if this SQL Source
+        is used within SQL File Resolvers.
+    aws_access_key_id
+        Optionally override the credentials using an AWS client id, must be
+        provided with a client secret override.
+    aws_access_key_secret
+        Optionally override the credentials using an AWS client secret, must
+        be provided with a client id override.
+    role_arn
+        Optionally override the credentials using an AWS role ARN.
+    aws_region
+        The AWS region code to connect to, such as `"us-east-1"`.
+    s3_staging_dir
+        The query result location to store query results within s3.
+    work_group
+        Optionally provide an Athena work group to query from.
+    catalog_name
+        The catalog name to query. Defaults to "default".
+    schema_name
+        The schema (database) name to query. Defaults to "awsdatacatalog".
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = AthenaSource(name="RISK")
+    """
+    ...
+
+
+def AthenaSource(
+    name: str | None = None,
+    aws_region: str | None = None,
+    aws_access_key_id: str | None = None,
+    aws_access_key_secret: str | None = None,
+    s3_staging_dir: str | None = None,
+    catalog_name: str | None = None,
+    schema_name: str | None = None,
+    role_arn: str | None = None,
+    work_group: str | None = None,
+    engine_args: Dict[str, Any] | None = None,
+) -> BaseSQLSourceProtocol:
+    """Create an Amazon Athena data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details. DynamoDBSources can be queried via PartiQL SQL
+    resolvers.
+
+    You may override the ambient AWS credentials by providing
+    either a client ID and secret, or a role ARN.
+    """
+    return AthenaSourceImpl(
+        name=name,
+        aws_region=aws_region,
+        aws_access_key_id=aws_access_key_id,
+        aws_access_key_secret=aws_access_key_secret,
+        s3_staging_dir=s3_staging_dir,
+        schema_name=schema_name,
+        catalog_name=catalog_name,
+        work_group=work_group,
+        role_arn=role_arn,
+        engine_args=engine_args,
+    )
+
+
+@overload
+def ClickhouseSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
+    """If you have only one Clickhouse connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = ClickhouseSource()
+    """
+    ...
+
+
+@overload
+def ClickhouseSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+    async_engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
+    """If you have only one Clickhouse integration, there's no need to provide
+    a distinguishing name.
+
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = ClickhouseSource(name="RISK")
+    """
+    ...
+
+
+@overload
+def ClickhouseSource(
+    *,
+    name: str | None = ...,
+    host: str,
+    port: Union[int, str] = ...,
+    db: str = ...,
+    user: str = ...,
+    password: str = ...,
+    use_tls: Union[bool, str] = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
+    async_engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
+    """
+    You can also configure the integration directly using environment
+    variables on your local machine or from those added through the
+    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
+
+    Parameters
+    ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
+    host
+        Name of host to connect to.
+    port
+        The port number to connect to at the server host.
+    db
+        The database name.
+    user
+        Clickhouse username to connect as.
+    password
+        The password to be used if the server demands password authentication.
+    use_tls
+        Whether to use tls protocol when communicating with the clickhouse engine, required for certain ports.
+        See https://clickhouse.com/docs/guides/sre/network-ports for more details. Defaults to True.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
+    async_engine_args:
+        Additional arguments to use when constructing an async SQLAlchemy engine.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> import os
+    >>> source = ClickhouseSource(
+    ...     host=os.getenv("CLICKHOUSE_HOST"),
+    ...     port=os.getenv("CLICKHOUSE_PORT"),
+    ...     db=os.getenv("CLICKHOUSE_DATABASE"),
+    ...     user=os.getenv("CLICKHOUSE_USER"),
+    ...     password=os.getenv("CLICKHOUSE_PASSWORD"),
+    ...     use_tls=os.getenv("CLICKHOUSE_USE_TLS"),
+    ... )
+    >>> from chalk.features import online
+    >>> @online
+    ... def resolver_fn() -> User.name:
+    ...     return source.query_string("select name from users where id = 4").one()
+    """
+    ...
+
+
+def ClickhouseSource(
+    *,
+    name: Optional[str] = None,
+    host: Optional[str] = None,
+    port: Optional[Union[int, str]] = None,
+    db: Optional[str] = None,
+    user: Optional[str] = None,
+    password: Optional[str] = None,
+    use_tls: Optional[Union[bool, str]] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
+    async_engine_args: Optional[Dict[str, Any]] = None,
+) -> BaseSQLSourceProtocol:
+    """Create a Clickhouse data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
+    return ClickhouseSourceImpl(
+        name=name,
+        host=host,
+        port=port,
+        db=db,
+        user=user,
+        password=password,
+        use_tls=use_tls,
+        engine_args=engine_args,
+        async_engine_args=async_engine_args,
+    )
+
+
+__all__ = (
+    "AthenaSource",
     "BaseSQLSourceProtocol",
     "BigQuerySource",
+    "ChalkQueryProtocol",
+    "ClickhouseSource",
     "CloudSQLSource",
     "DatabricksSource",
+    "DynamoDBSource",
+    "FinalizedChalkQuery",
+    "IncrementalSettings",
     "MySQLSource",
     "PostgreSQLSource",
     "RedshiftSource",
+    "SQLSourceGroup",
+    "SQLSourceWithTableIngestProtocol",
     "SQLiteFileSource",
     "SQLiteInMemorySource",
     "SnowflakeSource",
-    "TrinoSource",
-    "FinalizedChalkQuery",
-    "ChalkQueryProtocol",
+    "SpannerSource",
     "StringChalkQueryProtocol",
     "TableIngestProtocol",
-    "SQLSourceGroup",
-    "SQLSourceWithTableIngestProtocol",
-    "IncrementalSettings",
-]
+    "TrinoSource",
+    "make_sql_file_resolver",
+)
