@@ -161,6 +161,7 @@ class CommentDict(BaseModel):
     fields: Optional[Dict[str, str]]
     unique_on: Optional[List[str]]
     partitioned_by: Optional[List[str]]
+    skip_sql_validation: Optional[bool]
 
     @validator("tags", "environment", "unique_on", "partitioned_by", pre=True)
     @classmethod
@@ -492,23 +493,24 @@ def get_sql_file_resolver(
 
             if unrecognized_output:
                 value = output.split(".", 1)[1] if "." in output else output
-                assert glot_result.glot is not None, f"Failed to parse {glot_result.sql_string}"
-                try:
-                    error_builder.add_diagnostic_with_spellcheck(
-                        spellcheck_item=output,
-                        spellcheck_candidates=[
-                            feature.fqn
-                            for feature in registry_features[build_namespaced_name(name=parsed.namespace)].features
-                        ],
-                        message=message,
-                        code="153",
-                        label="output feature not recognized",
-                        range=error_builder.value_range_by_name(glot_result.glot, value),
-                    )
-                except Exception:
-                    # it's possible the graph is incomplete due to other import errors and we cannot find spellcheck candidates
-                    # in this case, we don't add a diagnostic because we are confident the error will surface later with a better stack
-                    pass
+                # Only add diagnostic if glot is available (i.e., validation was not skipped)
+                if glot_result.glot is not None:
+                    try:
+                        error_builder.add_diagnostic_with_spellcheck(
+                            spellcheck_item=output,
+                            spellcheck_candidates=[
+                                feature.fqn
+                                for feature in registry_features[build_namespaced_name(name=parsed.namespace)].features
+                            ],
+                            message=message,
+                            code="153",
+                            label="output feature not recognized",
+                            range=error_builder.value_range_by_name(glot_result.glot, value),
+                        )
+                    except Exception:
+                        # it's possible the graph is incomplete due to other import errors and we cannot find spellcheck candidates
+                        # in this case, we don't add a diagnostic because we are confident the error will surface later with a better stack
+                        pass
                 errors.append(
                     ResolverError(
                         display=message,
@@ -1167,6 +1169,20 @@ def _get_sql_glot(
             source=None,
         )
     assert source is not None, "unrecognized source should be handled by now"
+
+    # Skip SQL validation if the flag is set
+    if comment_dict_object.skip_sql_validation:
+        return GlotResult(
+            sql_string=sql_string,
+            glot=None,
+            args=args,
+            default_args=default_args,
+            comment_dict=comment_dict_object,
+            docstring=docstring,
+            errors=errors,
+            source=source,
+        )
+
     source_dialect_string = source.get_sqlglot_dialect()
     try:
         glots = sqlglot.parse(sql=sql_string, read=source_dialect_string)
@@ -1329,13 +1345,22 @@ def _parse_glot(
             errors=errors,
         )
 
-    assert isinstance(
-        glot_result.glot, (sqlglot.expressions.Select, sqlglot.expressions.Union)
-    ), f"glot {glot_result.glot} is not a select or union statement"
-    # sql target -> output feature string
-    fields: Dict[str, str] = {
-        column_name: f"{namespace}.{column_name}" for column_name in glot_result.glot.named_selects
-    }
+    # If SQL validation was skipped, we need to use the fields from comment_dict
+    if glot_result.glot is None:
+        # When validation is skipped, fields must be explicitly provided
+        if comment_dict.fields is not None:
+            fields: Dict[str, str] = {
+                variable: f"{namespace}.{output}" if "." not in output else output
+                for variable, output in comment_dict.fields.items()
+            }
+        else:
+            fields = {}
+    else:
+        assert isinstance(
+            glot_result.glot, (sqlglot.expressions.Select, sqlglot.expressions.Union)
+        ), f"glot {glot_result.glot} is not a select or union statement"
+        # sql target -> output feature string
+        fields = {column_name: f"{namespace}.{column_name}" for column_name in glot_result.glot.named_selects}
 
     return ParseResult(
         sql_string=glot_result.sql_string,
@@ -1599,6 +1624,7 @@ def make_sql_file_resolver(
     unique_on: Collection[FeatureReference] | None = None,
     partitioned_by: Collection[Any] | None = None,
     total: Optional[bool] = None,
+    skip_sql_validation: Optional[bool] = None,
 ):
     """Generate a Chalk SQL file resolver from a filepath and a sql string.
     This will generate a resolver in your web dashboard that can be queried,
@@ -1710,6 +1736,10 @@ def make_sql_file_resolver(
     total
         Whether this resolver returns all ids of a given namespace. To have this annotation, the resolver must
         take no arguments and return a DataFrame.
+    skip_sql_validation
+        If set to True, skips sqlglot validation of the SQL query. This is useful when the SQL contains
+        database-specific syntax that sqlglot cannot parse. When validation is skipped, you must provide
+        the `fields` parameter to explicitly map SQL columns to Chalk features.
 
 
     Examples
@@ -1771,6 +1801,7 @@ def make_sql_file_resolver(
         machine_type=machine_type,
         unique_on=[str(f) for f in unique_on] if unique_on is not None else None,
         partitioned_by=[str(f) for f in partitioned_by] if partitioned_by is not None else None,
+        skip_sql_validation=skip_sql_validation,
     )
     _GENERATED_SQL_FILE_RESOLVER_REGISTRY.add_sql_file_resolver(
         filepath=filename,
